--[[
	Boss Notes
	Attach notes to mobs and players
	Jug <jug@mangband.org>
-]]

-- Version id
local version = "Boss Notes v0.9.5"

-- Default UI position
cbn_ui_position = {x = 350, y = 350, w = 550, h = 250}

-- Default note data
cbn_note_data = {}

-- Windows transparency (0.0 is transparent, 1.0 is solid)
cbn_window_alpha = 0.3

-- Font sizes
cbn_normal_font = 16
cbn_title_font = 20

-- Default window lock status
cbn_locked = false

-- Setup UI
local context = UI.CreateContext("CBN")
local ui = UI.CreateFrame("Frame", "Main", context)
-- Window background
ui.background = UI.CreateFrame("Frame", "Background", ui)
ui.background:SetLayer(-1)
ui.background:SetPoint("TOPLEFT", ui, "TOPLEFT")
ui.background:SetPoint("BOTTOMRIGHT", ui, "BOTTOMRIGHT")
ui.background:SetBackgroundColor(0.0, 0.0, 0.0, cbn_window_alpha)
-- Lock/Unlock button
ui.lockbutton = UI.CreateFrame("Texture", "LockButton", ui)
ui.lockbutton:SetTexture("BossNotes", "lock.png")
ui.lockbutton:ResizeToTexture()
ui.lockbutton:SetPoint("TOPLEFT", ui, "TOPLEFT", 4, 4)
-- Edit button
ui.editbutton = UI.CreateFrame("Texture", "EditButton", ui)
ui.editbutton:SetTexture("BossNotes", "edit.png")
ui.editbutton:ResizeToTexture()
ui.editbutton:SetPoint("TOPLEFT", ui.lockbutton, "BOTTOMLEFT", 0, 4)
-- Title text
ui.title = UI.CreateFrame("Text", "TitleText", ui)
ui.title:SetText("Boss Notes")
ui.title:SetPoint("TOPLEFT", ui.lockbutton, "TOPRIGHT", 4, -4)
ui.title:SetPoint("RIGHT", ui, "RIGHT")
ui.title:SetFontSize(cbn_title_font)
ui.title:SetWordwrap(true)
-- Text block
ui.text = UI.CreateFrame("Text", "Text", ui)
ui.text:SetText("Drag this window to move it.\
Drag the bottom right corner to resize.\
Click the 'lock' icon (top left) to lock window position\
and make it click-through.  Click the 'edit' icon below the\
lock icon to edit the note.\
\
/cbn help \t-- display addon help")
ui.text:SetPoint("TOPLEFT", ui.title, "BOTTOMLEFT")
ui.text:SetPoint("BOTTOMRIGHT", ui, "BOTTOMRIGHT")
ui.text:SetFontSize(cbn_normal_font)
ui.text:SetWordwrap(true)
-- Text editor
ui.textedit = UI.CreateFrame("RiftTextfield", "TextEdit", ui)
ui.textedit:SetPoint("TOPLEFT", ui.title, "BOTTOMLEFT")
ui.textedit:SetPoint("BOTTOMRIGHT", ui, "BOTTOMRIGHT")
ui.textedit:SetVisible(false)
-- Save button
ui.save = UI.CreateFrame("Text", "SaveButton", ui)
ui.save:SetPoint("TOPRIGHT", ui, "TOPRIGHT", 2, 2)
ui.save:SetLayer(10)
ui.save:SetBackgroundColor(0.1, 0.1, 0.1, 0.5)
ui.save:SetText("Save")
ui.save:SetFontSize(cbn_normal_font)
ui.save:SetVisible(false)
-- Default size/position
ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 
	cbn_ui_position.x, cbn_ui_position.y)
ui:SetWidth(300)
ui:SetHeight(200)
-- Support movement
ui.moving = false
ui.resizing = false
ui.mx = 0
ui.my = 0
-- Unit in context
ui.current_target = nil
-- Mouse event handling
ui:SetMouseMasking("limited")

function onVariablesLoaded()
    if not cbn_ui_position or cbn_ui_position.x < 0 or cbn_ui_position.y < 0 
	or cbn_ui_position.w < 50 or cbn_ui_position.h < 20 then 
		cbn_ui_position = {x = 50, y = 50, w = 300, h = 200}
	end
	if not cbn_window_alpha then
		cbn_window_alpha = 0.3
	end
	if not cbn_normal_font then
		cbn_normal_font = 16
	end
	if not cbn_title_font then
		cbn_title_font = 20
	end
	if not cbn_locked then
		cbn_locked = false
	end
	-- Adjust Positions
	ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 
		cbn_ui_position.x, cbn_ui_position.y)
	ui:SetWidth(cbn_ui_position.w)
	ui:SetHeight(cbn_ui_position.h)
	ui.background:SetBackgroundColor(0.0, 0.0, 0.0, cbn_window_alpha)
	ui.text:SetFontSize(cbn_normal_font)
	ui.save:SetFontSize(cbn_normal_font)
	ui.title:SetFontSize(cbn_title_font)
	-- Set locked status
	ui.lockWindow(cbn_locked)
end

function ui.disableMouse()
	-- Hack out our mouse event handlers
	ui.Event["LeftDown"] = nil
	ui.Event["LeftUp"] = nil
	ui.Event["LeftUpoutside"] = nil
	ui.Event["MouseMove"] = nil
end

function ui.enableMouse()
	-- Setup our mouse event handlers
	ui.Event["LeftDown"] = LeftDown
	ui.Event["LeftUp"] = LeftUp
	ui.Event["LeftUpoutside"] = LeftUpoutside
	ui.Event["MouseMove"] = MouseMove
end

function ui.lockWindow(locked)
	if locked then
		ui.disableMouse()
	else
		ui.enableMouse()	
	end
end

function ui.lockbutton.Event:LeftDown()
	cbn_locked = not cbn_locked
	ui.lockWindow(cbn_locked)
end

function ui.editbutton.Event:LeftDown()
	ui.EditNote()
end

function LeftDown()
	if ui.lock then return end
	-- Where are we clicked?
	m = Inspect.Mouse()
	ui.x = ui:GetLeft()
	ui.y = ui:GetTop()
	ui.dx = m.x - ui.x
	ui.dy = m.y - ui.y
	-- Perhaps this is a resize attempt?
	if ui.dx > (ui:GetWidth()) - 30 
	and ui.dy > (ui:GetHeight()) - 30 then
		ui.resizing = true
	else
		ui.moving = true
	end
end

-- Edit a note
function ui.EditNote()
	-- Bail out if no current target
	if not ui.current_target then return end
	ui.text:SetVisible(false)
	ui.textedit:SetVisible(true)
	ui.textedit:SetKeyFocus(true)
	ui.save:SetVisible(true)
end

-- Save button clicked
function ui.save.Event:LeftDown()
	text = ui.textedit:GetText()
	if ui.current_target.player then
		key = ui.current_target.name
	else
		key = ui.current_target.name .. ui.current_target.healthMax
	end
	cbn_note_data[key] = text
	ui.text:SetText(text)
	ui.text:SetVisible(true)
	ui.textedit:SetVisible(false)
	ui.save:SetVisible(false)
end

-- Mouse button released
function LeftUp()
	ui.moving = false
	ui.resizing = false
end

-- Mouse button released outside our window
function LeftUpoutside()
	ui.moving = false
	ui.resizing = false
end

-- Mouse is moving
function MouseMove()
	if ui.moving then
		local x, y
		m = Inspect.Mouse()
		x = m.x - ui.dx
		y = m.y - ui.dy
		ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
		cbn_ui_position.x = x
		cbn_ui_position.y = y
	end
	if ui.resizing then
		local x, y
		m = Inspect.Mouse()
		x = m.x - ui.x
		y = m.y - ui.y
		if x > 30 and y > 30 then
			ui:SetWidth(x)
			ui:SetHeight(y)
			cbn_ui_position.w = x
			cbn_ui_position.h = y
		end
	end
end

-- Acquired a new target
local function onTargetChanged(targetid)
	if not targetid then 
		-- Clear window
		ui.text:SetText("")
		ui.title:SetText("")
		ui.textedit:SetText("")
		-- Forget target context
		ui.current_target = nil
		return
	end
	-- Grab all the unit info
	target = Inspect.Unit.Detail(targetid)
	-- Check for info on our target
	if target then
		-- Bail out if this target is already in context
		if ui.current_target and target.player then 
			if target.name == ui.current_target.name then
				return
			end
		else
			if ui.current_target
			and target.name == ui.current_target.name 
			and target.healthMax == ui.current_target.healthMax then
				return
			end
		end
		-- Grab notes on target, if any
		note = nil
		if cbn_note_data then
			if target.player then
				note = cbn_note_data[target.name]
			else
				note = cbn_note_data[target.name .. target.healthMax]
			end
		end
		if not note then note = "" end
		ui.title:SetText(target.name)
		ui.text:SetText(note)
		ui.textedit:SetText(note)
		ui.current_target = target
	end
end

-- Slash command handlers
local function slashCBN(param)
	if param == "" then
		ui:SetVisible( not ui:GetVisible() )
	elseif param == "help" then
		print(version)
		print("Click to target a mob then click the edit icon (top left) to")
		print("to edit the note.  Then the note will be displayed")
		print("every time you target the mob in the future.  Click the lock")
		print("icon to lock the window position and make it click-through.")
		print("")
		print("/cbn\t\tToggle window on or off")
		print("/cbn help\t\tShow this help!")
		print("/cbn clear\t\tClear the contents of the window")
		print("/cbn trans <1-10>\tSet window transparency, 1 to 10")
		print("/cbn font <size>\tSet font size, e.g. /cbn font 18")
		print("/cbn titlefont <size>\tSet title font size, e.g. /cbn titlefont 22")
	elseif param == "clear" then
		ui.title:SetText("")
		ui.text:SetText("")
	else
		-- Try commands of the form /cbn <keyword> <value>
		local keyword, value = param:match("(.+) (%d+)")
		value = tonumber(value)
		if keyword and value then
			if keyword == "trans" then
				cbn_window_alpha = 1 / value
				ui.background:SetBackgroundColor(0.0, 0.0, 0.0, cbn_window_alpha)
			elseif keyword == "font" then
				cbn_normal_font = value
				ui.text:SetFontSize(cbn_normal_font)
				ui.save:SetFontSize(cbn_normal_font)
			elseif keyword == "titlefont" then
				cbn_title_font = value
				ui.title:SetFontSize(cbn_title_font)			
			end
		end
	end

end

-- Initialise our event handlers
Library.LibUnitChange.Register("player.target")
table.insert(Event.LibUnitChange.player.target.Change, 
	{onTargetChanged, "BossNotes", "Player target changed"})
table.insert(Event.Addon.SavedVariables.Load.End, 
	{onVariablesLoaded, "BossNotes", "Saved vars loaded"})

-- Show our main window
ui:SetVisible(true)
ui.enableMouse()

-- Initialise slash commands
table.insert (Command.Slash.Register("cbn"), 
	{slashCBN, "BossNotes", "Toggle on or off"})
